using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using InfiniTec.Exchange;
using InfiniTec.Threading;
using System.IO;
using System.Threading;
using InfiniTec.Exchange.WebDav.Request;
using InfiniTec.Exchange.WebDav;

namespace ExampleApplication.Example
{
    [View("Attachment handling", "WebDAV low-level operations")]
    public partial class AttachmentsAsync : ExampleApplication.Example.ExampleBase
    {
        Item _Item;
        Stream _Stream;

        public AttachmentsAsync()
        {
            InitializeComponent();
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {

            _Item = CreateItem();
            _Item.Attachments.RefreshCompleted += Attachments_RefreshCompleted;
            _Item.Attachments.RefreshProgressChanged += Attachments_RefreshProgressChanged;

            _Item.Attachments.RefreshAsync();
        }

        void Attachments_RefreshProgressChanged(object sender, ProgressChangedEventArgs<RequestStatus> e)
        {
            Invoke((ThreadStart)delegate()
            {
                lbLog.Items.Add(e.Status.ToString());
            });
        }

        void Attachments_RefreshCompleted(object sender, AsyncCompletedEventArgs e)
        {
            ListViewItem item;

            Invoke((ThreadStart)delegate()
           {
               if (e.Error != null)
               {
                   ViewManager.ShowException(e.Error);
                   return;
               }

               lvAttachments.Items.Clear();
               lbLog.Items.Add("Found " + _Item.Attachments.Count + " attachments");
               foreach (Attachment attachment in _Item.Attachments)
               {
                   item = new ListViewItem(attachment.Name);
                   item.SubItems.Add(attachment.Size.ToString());
                   item.SubItems.Add(attachment.MimeType);
                   item.SubItems.Add(attachment.ContentDisposition);
                   item.Tag = attachment;

                   lvAttachments.Items.Add(item);
               }             
               UpdateButtons();
           });
        }

        private void UpdateButtons()
        {
            btnDelete.Enabled = lvAttachments.SelectedItems.Count > 0;
            btnDownload.Enabled = lvAttachments.SelectedItems.Count > 0;
        }

        protected Item CreateItem()
        {
            Item item = new Item(RootPath, ConnectionView.Connection);

            return item;
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            Attachment attachment;

            attachment = lvAttachments.SelectedItems[0].Tag as Attachment;

            if (attachment == null) return;

            if (saveFileDialog.ShowDialog() != DialogResult.OK) return;

            attachment.DownloadCompleted += attachment_DownloadCompleted;
            attachment.DownloadProgressChanged += attachment_DownloadProgressChanged;

            _Stream = new FileStream(saveFileDialog.FileName, FileMode.Create, FileAccess.Write, FileShare.Read);

            attachment.DownloadAsync(_Stream);
        }

        void attachment_DownloadProgressChanged(object sender, ProgressChangedEventArgs<RequestStatus> e)
        {
            Invoke((ThreadStart)delegate()
           {
               lbLog.Items.Add(e.Status.ToString());
           });
        }

        void attachment_DownloadCompleted(object sender, AsyncCompletedEventArgs e)
        {
            Stream stream;

            Invoke((ThreadStart)delegate()
           {
               stream = Interlocked.Exchange<Stream>(ref _Stream, null);
               if (stream != null) stream.Close();

               if (e.Error != null) ViewManager.ShowException(e.Error);

               lbLog.Items.Add("Finished dowloading the attachment");
           });
        }

        private void lvAttachments_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateButtons();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() != DialogResult.OK) return;

            _Item = CreateItem();
            _Item.Attachments.AddCompleted += Attachments_AddCompleted;
            _Item.Attachments.AddProgressChanged += Attachments_AddProgressChanged;

            _Stream = File.OpenRead(openFileDialog.FileName);

            _Item.Attachments.AddAsync(new FileInfo(openFileDialog.FileName).Name, _Stream);
            
        }

        void Attachments_AddProgressChanged(object sender, ProgressChangedEventArgs<RequestStatus> e)
        {
            Invoke((ThreadStart)delegate()
           {
               lbLog.Items.Add(e.Status.ToString());
           });
        }

        void Attachments_AddCompleted(object sender, AsyncCompletedEventArgs e)
        {
            Stream stream;

            Invoke((ThreadStart)delegate()
           {
               stream = Interlocked.Exchange<Stream>(ref _Stream, null);
               if (stream != null) stream.Close();

               if (e.Error != null) ViewManager.ShowException(e.Error);

               lbLog.Items.Add("Finished adding the attachment");
           });
        }
    }
}

