using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using InfiniTec.Exchange;
using InfiniTec.Threading;
using System.Threading;
using InfiniTec.Exchange.WebDav.Request;
using InfiniTec.Exchange.WebDav;

namespace ExampleApplication.Example
{
    [View("Set property value", "WebDAV low-level operations")]
    public partial class SetPropertyValue : ExampleApplication.Example.ExampleBase
    {
        public SetPropertyValue()
        {
            InitializeComponent();
        }

        private void btnCommit_Click(object sender, EventArgs e)
        {
            Item item;
            string name;

            name = cbNames.Text;

            item = GetItem();

            switch (cbTypes.Items[cbTypes.SelectedIndex].ToString())
            {
                case "string": item.Properties.Add<string>(name, tbValue.Text); break;
                case "boolean": item.Properties.Add<bool>(name, bool.Parse(tbValue.Text)); break;
                case "int": item.Properties.Add<int>(name, int.Parse(tbValue.Text)); break;
                case "datetime": item.Properties.Add<DateTime>(name, DateTime.Parse(tbValue.Text)); break;
            }
            item.SaveCompleted += item_SaveCompleted;
            item.SaveProgressChanged += item_SaveProgressChanged;

            item.SaveAsync();
        }

        void item_SaveProgressChanged(object sender, ProgressChangedEventArgs<RequestStatus> e)
        {
            Invoke((ThreadStart)delegate
            {
                lbLog.Items.Add(e.Status.ToString());
            });
        }

        void item_SaveCompleted(object sender, AsyncCompletedEventArgs e)
        {
            Invoke((ThreadStart)delegate
           {
               if (e.Error != null)
               {
                   ViewManager.ShowException(e.Error);
                   return;
               }

               lbLog.Items.Add("Finished. Cancelled: " + e.Cancelled);
           });
        }

        protected Item GetItem()
        {
            Item item = new Item(RootPath, ConnectionView.Connection);

            return item;
        }

        private void SetPropertyValue_Load(object sender, EventArgs e)
        {
            FillProperties(typeof(NamedProperties));
            cbTypes.SelectedIndex = 0;
        }

        private void FillProperties(Type type)
        {
            foreach (FieldInfo info in type.GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                cbNames.Items.Add(info.GetValue(null));
            }
            foreach (Type t in type.GetNestedTypes())
            {
                FillProperties(t);
            }
        }
    }
}


