using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using InfiniTec.Exchange;
using System.Security.Principal;
using InfiniTec.Security;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Exchange.Security;

namespace ExampleApplication.Example
{
    [View("Update security descriptor", "WebDAV low-level operations")]
    public partial class UpdateSecurityDescriptor: ExampleApplication.Example.ExampleBase
    {
        Folder _Folder;
        FolderSecurity _Security;

        public UpdateSecurityDescriptor()
        {
            InitializeComponent();
        }

        private void UpdateSecurityDescriptor_Load(object sender, EventArgs e)
        {
            foreach (string entry in Enum.GetNames(typeof(AccessRole)))
            {
                cbRoles.Items.Add(Enum.Parse(typeof(AccessRole), entry));
            }
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            IdentityResolver id;
            FolderAce ace;

            if (listView1.SelectedItems.Count == 0) return;
 
            ace = listView1.SelectedItems[0].Tag as FolderAce;

            if (ace == null) return;

            id = new IdentityResolver(ace.IdentityReference, ConnectionView.Connection.Server);
            tbName.Text = id.Account.Value;
            cbRoles.SelectedValue = ace.Role;
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            _Folder = new Folder(RootPath, ConnectionView.Connection);
            _Folder.Refresh();

            _Security = _Folder.GetSecurity();

            UpdateListview();
        }

        private void UpdateListview()
        {
            listView1.Items.Clear();
            foreach (FolderAce ace in _Security.GetAccessControlEntries())
            {
                AddAce(ace);
            }
        }

        private void AddAce(FolderAce entry)
        {
            ListViewItem item;
            IdentityResolver account;

            account = new IdentityResolver(entry.IdentityReference, ConnectionView.Connection.Server);
            item = new ListViewItem();
            item.Text = account.Account.Value;
            item.SubItems.Add(account.AccountType.ToString());
            item.Tag = entry;
            
            item.SubItems.Add(((FolderAce) entry).Role.ToString());

            listView1.Items.Add(item);
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            IdentityResolver identity;

            identity = new IdentityResolver(new NTAccount(tbName.Text),
                ConnectionView.Connection.Server);
            

            _Security.SetAccess(identity.Sid, (AccessRole) cbRoles.SelectedItem);

            UpdateListview();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 0) return;

            FolderAce ace = listView1.SelectedItems[0].Tag as FolderAce;
            _Security.RemoveAccess(ace.IdentityReference);
            listView1.Items.Remove(listView1.SelectedItems[0]);
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            _Folder.SetSecurity(_Security);
            _Folder.Save();
        }

    }
}

