using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Threading;

namespace ExampleApplication
{
    public partial class ViewManager: Form
    {
        public ViewManager()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                lvExamples.Items.Add(CreateView<ConnectionView>(lvExamples.Groups["Settings"]));

                AddExamples();
            }
            finally
            {
                lvExamples.EndUpdate();
            }
        }

        private void AddExamples()
        {
            Assembly assembly;
            List<ListViewItem> items;

            assembly = Assembly.GetExecutingAssembly();

            items = new List<ListViewItem>();

            foreach (Type type in assembly.GetTypes())
            {
                if (!type.IsSubclassOf(typeof(UserControl))) continue;
                if (!ViewAttribute.IsDefined(type)) continue;

                if (!ViewAttribute.Get(type).AddAutomatically) continue;

                items.Add(CreateView(type));
            }

            items.Sort(delegate(ListViewItem a, ListViewItem b)
            {
                if (a == b) return 0;
                if (a == null) return -1;
                if (b == null) return 1;
                return a.Text.CompareTo(b.Text);
            });

            lvExamples.Items.AddRange(items.ToArray());
        }

        private ListViewItem CreateView(Type type)
        {
            ListViewGroup group;
            ViewAttribute attribute;

            if (!ViewAttribute.IsDefined(type)) throw new ArgumentException("Specified type is not a view");

            attribute = ViewAttribute.Get(type);

            group = lvExamples.Groups[attribute.Category];
            if (group == null) group = lvExamples.Groups.Add(attribute.Category, attribute.Category);


            return CreateView(type, group);
        }

        private ListViewItem CreateView(Type type, ListViewGroup group)
        {
            ViewAttribute attribute;
            ListViewItem item;

            if (!ViewAttribute.IsDefined(type)) throw new ArgumentException("Specified type is not a view");

            UserControl control = (UserControl) Activator.CreateInstance(type);


            if (control.Size.Width + 200 > ClientSize.Width) Width = control.Size.Width + 200;

            if (control.Size.Height + 50 > ClientSize.Height) Height = control.Height + 80;
            
            attribute = ViewAttribute.Get(type);


            item = new ListViewItem(attribute.Name);
            item.Tag = control;
            item.Group = group; 

            return item;
        }

        private ListViewItem CreateView<T>(ListViewGroup group) where T: UserControl
        {
            return CreateView(typeof(T), group);
        }

        public static void ShowException(Exception ex)
        {
            using (ExceptionDialog dlg = new ExceptionDialog())
            {
                dlg.Exception = ex;
                dlg.ShowDialog();
            }
        }

        private void lvExamples_SelectedIndexChanged(object sender, EventArgs e)
        {
            UserControl control;
            ListViewItem item;

            if (lvExamples.SelectedItems.Count == 0) return;

            item = lvExamples.SelectedItems[0];

            if (item.Tag == null) return;

            try
            {
                SuspendLayout();

                ViewContainer.Controls.Clear();

                control = (UserControl) item.Tag;
                control.Dock = DockStyle.Fill;

                ViewContainer.Controls.Add(control);
                ViewTitle.Text =
                    ViewAttribute.Get(control.GetType()).Name;
            }
            finally
            {
                ResumeLayout();
            }
        }
    }
}